@@ -12,7 +12,7 @@ use Math::Prime::Util qw/ prime_get_config
 
 BEGIN {
   $Math::Prime::Util::RandomPrimes::AUTHORITY = 'cpan:DANAJ';
-  $Math::Prime::Util::RandomPrimes::VERSION = '0.42';
+  $Math::Prime::Util::RandomPrimes::VERSION = '0.43';
 }
 
 BEGIN {
@@ -296,6 +296,17 @@ my $_random_prime = sub {
     $low = Math::BigInt->new("$low") if ref($high) eq 'Math::BigInt';
     confess "Invalid _random_prime parameters: $low, $high" if ($low % 2) == 0 || ($high % 2) == 0;
 
+    # If they want to use the crappy PRIMEINC algorithm, do it.
+    if (prime_get_config->{'use_primeinc'}) {
+      $prime = $low + $_RANDF->($high-$low);   # Random number from low-high
+      $prime = next_prime($prime-1);           # Increment until prime
+      if ($prime > $high) {                    # Rollover
+        $prime = next_prime($low-1);
+        return if $prime > $high;
+      }
+      return $prime;
+    }
+
     # We're going to look at the odd numbers only.
     my $oddrange = (($high - $low) >> 1) + 1;
 
@@ -573,6 +584,22 @@ sub random_nbit_prime {
   croak "Mid-size random primes not supported on broken old Perl"
     if OLD_PERL_VERSION && MPU_64BIT && $bits > 49 && $bits <= 64;
 
+  # If they want to use the crappy PRIMEINC algorithm, do it.
+  if (prime_get_config->{'use_primeinc'}) {
+    my($prime, $low, $high);
+    $low  = ($bits >  MPU_MAXBITS) ? Math::BigInt->bone->blsft($bits-1)
+                                   : (1 << ($bits-1));
+    $high = ($bits >  MPU_MAXBITS) ? Math::BigInt->bone->blsft($bits)->bdec()
+          : ($bits == MPU_MAXBITS) ? ~0
+                                   : (1 << $bits) - 1;
+    $prime = next_prime($low + $_RANDF_NBIT->($bits-1) - 1);
+    if ($prime > $high) {                    # Rollover
+      $prime = next_prime($low-1);
+      return if $prime > $high;
+    }
+    return $prime;
+  }
+
   # Fouque and Tibouchi (2011) Algorithm 1 (basic)
   # Modified to make sure the nth bit is always set.
   #
@@ -1073,7 +1100,7 @@ Math::Prime::Util::RandomPrimes - Generate random primes
 
 =head1 VERSION
 
-Version 0.42
+Version 0.43
 
 
 =head1 SYNOPSIS
